/****************************************************************************\
 *           Copyright (C) 2012 Texas Instruments Incorporated.             *
 *                           All Rights Reserved                            *
 *                                                                          *
 * GENERAL DISCLAIMER                                                       *
 * ------------------                                                       *
 * All software and related documentation is provided "AS IS" and without   *
 * warranty or support of any kind and Texas Instruments expressly disclaims*
 * all other warranties, express or implied, including, but not limited to, *
 * the implied warranties of merchantability and fitness for a particular   *
 * purpose.  Under no circumstances shall Texas Instruments be liable for   *
 * any incidental, special or consequential damages that result from the    *
 * use or inability to use the software or related documentation, even if   *
 * Texas Instruments has been advised of the liability.                     *
 ****************************************************************************
This example shows three main features of Timers:
1. generate one-shot pulse and interrupt
2. generate continual clock and interrupts
3. watch-dog
 ****************************************************************************
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            Jan 5, 2012                                                 *
***************************************************************************/
#include <c6x.h>
#include <stdio.h>
#include <csl_bootcfgAux.h>
#include "Keystone_common.h"

/*delay (in millisecond) before the timer generate pulse*/
#define PULSE_DELAY_MS 			2000
/*period (in millisecond) of the clock generated by timer*/
#define CLOCK_PERIOD_MS 		1
/*number of the clocks generated in this test*/
#define NUM_CLOCKS_GENERATED 	500
/*period (in millisecond) of the watch-dog timer*/
#define WATCH_DOG_PERIOD_MS 	2000

/*since the timer runs at (DSP core clock)/6
the conversion between millisecond and timer PRD value is:
PRD = (xxx_MS/1000)*DSP_CLK_HZ/6
    = xxx_MS*(DSP_CLK_HZ/1000)/6
    = xxx_MS*DSP_CLK_KHZ/6*/
    
volatile Uint32 uiTick = 0;

void Timer_Interrupts_Init(void)
{
	CGEM_regs->INTMUX1 = 0;
	CGEM_regs->INTMUX2 = 0;
	//map local timer interrupt to INT14
	CGEM_regs->INTMUX3 = CSL_GEM_TINTLN<<CSL_CGEM_INTMUX3_INTSEL14_SHIFT;

	 /*Clear all DSP core events*/
	CGEM_regs->EVTCLR[0]= 	0xFFFFFFFF;
	CGEM_regs->EVTCLR[1]= 	0xFFFFFFFF;
	CGEM_regs->EVTCLR[2]= 	0xFFFFFFFF;
	CGEM_regs->EVTCLR[3]= 	0xFFFFFFFF;

	//clear DSP core interrupt flag
	ICR= IFR;

	//enable INT14
	IER = IER|(1<<14);

	/*Interrupt Service Table Pointer to begining of LL2 memory*/
	ISTP= 0x800000;

	//enable GIE
	TSR = TSR|1;
}

//Interrupt service routine for timer
void interrupt Timer_ISR(void)
{
	uiTick++;
}

/*use local timer (==DNUM of a core) to generate a pulse on TIMO0,
a interrupt is generated as well*/
void generate_pulse_by_timer()
{
	puts("\na pulse will be generated on TIMO0...");

	uiTick = 0;

	//select output on TIMO0 from local timer.
	boot_cfg_regs->TOUTSEL = (DNUM*2)<<CSL_BOOTCFG_TOUTSEL_TOUTSEL0_SHIFT;

	//configure the timer for one shot pulse
	Timer_One_Shot_Init(DNUM, PULSE_DELAY_MS*CPU_CLK_KHZ/6);

	//wait until the interupt happen, which means the pulse has been generated
	while(0==uiTick)
		asm(" nop 5");

	puts("a pulse was generated on TIMO0\n");
}

/*use local timer (==DNUM of a core) to generate a clock on TIMO0,
interrupts are generated as well*/
void generate_clocks_by_timer()
{
	puts("clocks will be generated on TIMO0...");

	uiTick = 0;

	//select output on TIMO0 from local timer.
	boot_cfg_regs->TOUTSEL = (DNUM*2)<<CSL_BOOTCFG_TOUTSEL_TOUTSEL0_SHIFT;

	/*configure the timer to generate clocks and interrupts
	please note, two timer periods generate a clock, one timer period output
	high voltage level, the other timer period output low voltage level,
	so, the timer period should be half ot the desired clock period*/
	Timer_Init(DNUM, CLOCK_PERIOD_MS*CPU_CLK_KHZ/6/2);

	//wait until a number of interupts happen
	while(uiTick<NUM_CLOCKS_GENERATED*2)
		asm(" nop 5");
	Reset_Timer(DNUM); 	//stop the timer
	
	puts("clocks were generated on TIMO0\n");
}

/*set local timer (==DNUM of a core) as watch dog timer,
service the watch-dog for sometime, 
after stop servicing watch-dog, let it trigger NMI*/
void watchdog_timer_test()
{
	int i;
	Uint32 timerCnt;
	
	//select output on TIMO0 from local timer.
	boot_cfg_regs->TOUTSEL = (DNUM*2)<<CSL_BOOTCFG_TOUTSEL_TOUTSEL0_SHIFT;

	boot_cfg_regs->RSTMUX[DNUM] = 
		(4<<CSL_BOOTCFG_RSTMUX0_RSTMUX_DELAY0_SHIFT)
		/*WD timer event trigger NMI*/
		|(WD_TRIGGER_NMI<<CSL_BOOTCFG_RSTMUX0_RSTMUX_OMODE0_SHIFT);

	puts("start watch-dog timer...");

	//configure the timer for watch-dog
	Watchdog_Timer_Init(DNUM, WATCH_DOG_PERIOD_MS*CPU_CLK_KHZ/6);

	//service the watch-dog for sometime, 
	for(i=1; i<=50; i++)
	{
		timerCnt= timerRegs[DNUM]->CNTLO;
		Service_Watchdog(DNUM);
		printf("service watch-dog %d times, at time counter = %d\n", 
			i, timerCnt);
	}

	puts("stop servicing watch-dog, it will timeout and trigger NMI...\n");
}

main()
{
	TDSP_Type dspType;

	CSL_BootCfgUnlockKicker();

	dspType= Get_dsp_type();
	if(TCI6608==dspType)
	{
		//DSP core speed
		//KeyStone_main_PLL_init (20, 1); 	//for 50MHz input clock
		KeyStone_main_PLL_init (10, 1); 	//for 100MHz input clock
	}
	else if(TCI6616==dspType||TCI6618==dspType||TCI6614==dspType)
	{
		//DSP core speed
		KeyStone_main_PLL_init (236, 29); 	//for 122.88MHz input clock
	}
	else
	{
		puts("Unknown DSP type!");
		return 1;
	}

	Timer_Interrupts_Init();
	Exception_cfg();
	
    CACHE_setL1PSize(CACHE_L1_32KCACHE);
    CACHE_setL1DSize(CACHE_L1_32KCACHE);
    CACHE_setL2Size(CACHE_0KCACHE);
	CACHE_invAllL1p(CACHE_WAIT);
	CACHE_wbInvAllL1d(CACHE_WAIT);

	generate_pulse_by_timer();
	generate_clocks_by_timer();
	watchdog_timer_test();
	
	while(1);
}

